from django.http import Http404, HttpResponse
from django.test import RequestFactory, TestCase, override_settings
from mock import Mock, patch

from wagtail.wagtailcore.models import Site

from wagtailsharing.models import SharingSite
from wagtailsharing.tests.helpers import create_draft_page
from wagtailsharing.views import ServeView


class TestServeView(TestCase):
    def setUp(self):
        self.factory = RequestFactory()
        self.default_site = Site.objects.get(is_default_site=True)

    def make_request(self, path, **kwargs):
        request = self.factory.get(path, **kwargs)
        request.site = self.default_site
        return request

    def create_sharing_site(self, hostname):
        SharingSite.objects.create(
            site=self.default_site,
            hostname=hostname
        )

    def assert_title_matches(self, response, title):
        self.assertContains(response, title)

    def test_no_sharing_site_exists_uses_wagtail_serve(self):
        request = self.make_request('/')
        with patch('wagtailsharing.views.wagtail_serve') as wagtail_serve:
            ServeView().get(request, request.path)
            wagtail_serve.assert_called_once_with(request, request.path)

    def test_default_site_missing_page_raises_404(self):
        self.create_sharing_site(hostname='hostname')

        request = self.make_request('/missing/')
        with self.assertRaises(Http404):
            ServeView().get(request, request.path)

    def test_sharing_site_missing_page_raises_404(self):
        self.create_sharing_site(hostname='hostname')

        request = self.make_request('/missing/', HTTP_HOST='hostname')
        with self.assertRaises(Http404):
            ServeView().get(request, request.path)

    def test_default_site_unpublished_page_raises_404(self):
        self.create_sharing_site(hostname='hostname')
        create_draft_page(self.default_site, title='unpublished')

        request = self.make_request('/unpublished/')
        with self.assertRaises(Http404):
            ServeView().get(request, request.path)

    def test_sharing_site_unpublished_page_returns_200(self):
        self.create_sharing_site(hostname='hostname')
        create_draft_page(self.default_site, title='draft')

        request = self.make_request('/draft/', HTTP_HOST='hostname')
        response = ServeView().get(request, request.path)
        self.assertEqual(response.status_code, 200)

    def test_default_site_published_page_returns_200(self):
        self.create_sharing_site(hostname='hostname')
        page = create_draft_page(self.default_site, title='published')
        page.save_revision().publish()

        request = self.make_request('/published/')
        response = ServeView().get(request, request.path)
        self.assertEqual(response.status_code, 200)

    def test_sharing_site_published_page_returns_200(self):
        self.create_sharing_site(hostname='hostname')
        page = create_draft_page(self.default_site, title='published')
        page.save_revision().publish()

        request = self.make_request('/published/', HTTP_HOST='hostname')
        response = ServeView().get(request, request.path)
        self.assertEqual(response.status_code, 200)

    def test_default_site_draft_version_returns_published_version(self):
        self.create_sharing_site(hostname='hostname')
        page = create_draft_page(self.default_site, title='original')
        page.save_revision().publish()
        page.title = 'changed'
        page.save_revision()

        request = self.make_request('/original/')
        response = ServeView().get(request, request.path)
        self.assert_title_matches(response, 'original')

    def test_sharing_site_draft_version_returns_draft_version(self):
        self.create_sharing_site(hostname='hostname')
        page = create_draft_page(self.default_site, title='original')
        page.save_revision().publish()
        page.title = 'changed'
        page.save_revision()

        request = self.make_request('/original/', HTTP_HOST='hostname')
        response = ServeView().get(request, request.path)
        self.assert_title_matches(response, 'changed')

    def test_before_serve_page_hook_called(self):
        self.create_sharing_site(hostname='hostname')
        create_draft_page(self.default_site, title='page')

        with patch(
            'wagtail.wagtailcore.hooks.get_hooks'
        ) as get_hooks:
            request = self.make_request('/page/', HTTP_HOST='hostname')
            ServeView().get(request, request.path)
            get_hooks.assert_called_once_with('before_serve_page')

    def test_before_serve_page_hook_returns_redirect(self):
        self.create_sharing_site(hostname='hostname')
        create_draft_page(self.default_site, title='page')

        with patch(
            'wagtail.wagtailcore.hooks.get_hooks',
            return_value=[Mock(return_value=HttpResponse(status=999))]
        ):
            request = self.make_request('/page/', HTTP_HOST='hostname')
            response = ServeView().get(request, request.path)
            self.assertEqual(response.status_code, 999)

    @override_settings(WAGTAILSHARING_BANNER=False)
    def test_no_banner_setting(self):
        response = Mock(content='<body>abcde</body>')
        response = ServeView.postprocess_response(response)
        self.assertEqual(response.content, '<body>abcde</body>')

    @override_settings(WAGTAILSHARING_BANNER=True)
    def test_banner_setting_no_body(self):
        response = Mock(content='abcde')
        response = ServeView.postprocess_response(response)
        self.assertEqual(response.content, 'abcde')

    @override_settings(WAGTAILSHARING_BANNER=True)
    def test_banner_setting_modified_body(self):
        response = Mock(content='<body>abcde</body>')
        response = ServeView.postprocess_response(response)
        self.assertIn('wagtailsharing-banner', response.content)



from __future__ import absolute_import, unicode_literals

from django.db import models
from django.utils.encoding import python_2_unicode_compatible
from wagtail.wagtailcore.models import Site


@python_2_unicode_compatible
class SharingSite(models.Model):
    site = models.OneToOneField(Site, on_delete=models.CASCADE,
                                related_name='sharing_site')
    hostname = models.CharField(max_length=255, db_index=True)
    port = models.IntegerField(default=80)

    class Meta:
        unique_together = ('hostname', 'port')

    def __str__(self):
        return(
            self.hostname +
            ('' if self.port == 80 else (':%d' % self.port))
        )

    @classmethod
    def find_for_request(cls, request):
        """
        Find a shared site associated with this HTTP request object. Raises
        Raises SharingSite.DoesNotExist if no appropriate sharing site exists.

        Uses request hostname and port to find matching sharing site.
        """
        try:
            hostname = request.get_host().split(':')[0]
        except KeyError:
            hostname = None

        try:
            port = request.get_port()
        except (AttributeError, KeyError):
            port = request.META.get('SERVER_PORT')

        return cls.objects.get(hostname=hostname, port=port)

    @property
    def root_url(self):
        if self.port == 80:
            return 'http://{}'.format(self.hostname)
        elif self.port == 443:
            return 'https://{}'.format(self.hostname)
        else:
            return 'http://{}:{:d}'.format(self.hostname, self.port)



from django.test import TestCase
from mock import Mock, patch
from wagtail.tests.testapp.models import SimplePage

from wagtailsharing.wagtail_hooks import add_sharing_link


class TestAddSharingLink(TestCase):
    def setUp(self):
        self.page = SimplePage(title='title', slug='slug', content='content')
        self.page_perms = Mock()

    def test_no_link_no_button(self):
        with patch(
            'wagtailsharing.wagtail_hooks.get_sharing_url',
            return_value=None
        ):
            links = add_sharing_link(self.page, self.page_perms)
            self.assertFalse(list(links))

    def check_link_makes_button(self):
        url = 'http://test.domain/slug/'
        with patch(
            'wagtailsharing.wagtail_hooks.get_sharing_url',
            return_value=url
        ):
            links = add_sharing_link(self.page, self.page_perms)
            button = next(links)
            self.assertEqual(button.url, url)
            self.assertIn(self.page.title, button.attrs['title'])

    def test_link_makes_button(self):
        self.check_link_makes_button()

    def test_link_no_admin_display_title(self):
        try:
            delattr(self.page, 'get_admin_display_title')
        except AttributeError:
            pass

        self.check_link_makes_button()

    def test_link_admin_display_title(self):
        if not hasattr(self.page, 'get_admin_display_title'):
            self.page.get_admin_display_title = lambda: self.page.title

        self.check_link_makes_button()



from __future__ import unicode_literals

import inspect

from bs4 import BeautifulSoup
from django.conf import settings
from django.http import Http404, HttpResponse
from django.template import loader
from django.views.generic import View
from wagtail.wagtailcore import hooks
from wagtail.wagtailcore.views import serve as wagtail_serve

from wagtailsharing.models import SharingSite


class ServeView(View):
    def get(self, request, path):
        try:
            sharing_site = SharingSite.find_for_request(request)
        except SharingSite.DoesNotExist:
            return wagtail_serve(request, path)

        page, args, kwargs = self.get_requested_page(
            sharing_site.site,
            request,
            path
        )

        return self.serve_latest_revision(page, request, args, kwargs)

    @staticmethod
    def get_requested_page(site, request, path):
        """Retrieve a page from a site given a request and path.

        This method uses the standard `wagtail.wagtailcore.Page.route` method
        to retrieve a page using its path from the given site root.

        If a requested page exists and is published, the result of `Page.route`
        can be returned directly.

        If the page exists but is not yet published, `Page.route` raises an
        `Http404`, which this method tries to catch and handle. `Page.route`
        raises `Http404` in two cases: if no page with the given path exists
        and if a page exists but is unpublished. This method catches both
        cases, and, if they fall into the latter category, returns the
        requested page back to the caller despite its draft status.
        """
        path_components = [
            component for component in path.split('/') if component
        ]

        try:
            return site.root_page.route(request, path_components)
        except Http404:
            exception_source = inspect.trace()[-1]
            stack_frame = exception_source[0]

            page = stack_frame.f_locals['self']
            path_components = stack_frame.f_locals['path_components']

            if path_components:
                raise

            return page, [], {}

    @classmethod
    def serve_latest_revision(cls, page, request, args, kwargs):
        # Call the before_serve_page hook.
        for fn in hooks.get_hooks('before_serve_page'):
            result = fn(page, request, args, kwargs)
            if isinstance(result, HttpResponse):
                return result

        # Get the latest revision for the requested page.
        page = page.get_latest_revision_as_page()

        # Generate the page response.
        response = page.serve(request, *args, **kwargs)

        # Do appropriate response postprocessing.
        response = cls.postprocess_response(response)

        return response

    @classmethod
    def postprocess_response(cls, response):
        if getattr(settings, 'WAGTAILSHARING_BANNER', True):
            cls.add_response_banner(response)

        return response

    @staticmethod
    def add_response_banner(response):
        response.render()

        html = BeautifulSoup(response.content, 'html.parser')

        banner_template_name = 'wagtailsharing/banner.html'
        banner_template = loader.get_template(banner_template_name)
        banner_html = banner_template.render()

        banner = BeautifulSoup(banner_html, 'html.parser')

        if html.body:
            html.body.insert(0, banner)
            response.content = html.prettify()

        return response



from __future__ import absolute_import, unicode_literals

from django.conf.urls import include, url
from wagtail.wagtailadmin import urls as wagtailadmin_urls

from wagtailsharing import urls as wagtailsharing_urls


urlpatterns = [
    url(r'^admin/', include(wagtailadmin_urls)),
    url(r'', include(wagtailsharing_urls)),
]



from django.test import TestCase
from wagtail.tests.testapp.models import SimplePage
from wagtail.wagtailcore.models import Site

from wagtailsharing.helpers import get_sharing_url
from wagtailsharing.models import SharingSite
from wagtailsharing.tests.helpers import create_draft_page


class TestGetSharingUrl(TestCase):
    def setUp(self):
        self.default_site = Site.objects.get(is_default_site=True)

    def create_sharing_site(self, hostname):
        SharingSite.objects.create(
            site=self.default_site,
            hostname=hostname
        )

    def test_unroutable_page_no_sharing_site_returns_none(self):
        page = SimplePage(title='title', slug='slug', content='content')
        self.assertIsNone(get_sharing_url(page))

    def test_unroutable_page_sharing_site_returns_none(self):
        self.create_sharing_site(hostname='hostname')
        page = SimplePage(title='title', slug='slug', content='content')
        self.assertIsNone(get_sharing_url(page))

    def test_draft_page_no_sharing_site_returns_none(self):
        page = create_draft_page(self.default_site, title='draft')
        self.assertIsNone(get_sharing_url(page))

    def test_draft_page_sharing_site_returns_url(self):
        self.create_sharing_site(hostname='hostname')
        page = create_draft_page(self.default_site, title='draft')
        self.assertEqual(get_sharing_url(page), 'http://hostname/draft/')

    def test_published_page_no_sharing_site_returns_none(self):
        page = create_draft_page(self.default_site, title='published')
        page.save_revision().publish()
        self.assertIsNone(get_sharing_url(page))

    def test_published_page_sharing_site_returns_url(self):
        self.create_sharing_site(hostname='hostname')
        page = create_draft_page(self.default_site, title='published')
        page.save_revision().publish()
        self.assertEqual(get_sharing_url(page), 'http://hostname/published/')



from django.utils.text import slugify
from wagtail.tests.testapp.models import SimplePage


def create_draft_page(site, title):
    page = SimplePage(
        title=title,
        slug=slugify(title),
        content='content',
        live=False
    )

    site.root_page.add_child(instance=page)
    return page



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import django.db.models.deletion

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('wagtailcore', '0030_index_on_pagerevision_created_at'),
    ]

    operations = [
        migrations.CreateModel(
            name='SharingSite',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('hostname', models.CharField(max_length=255, db_index=True)),
                ('port', models.IntegerField(default=80)),
                ('site', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, related_name='sharing_site', to='wagtailcore.Site')),
            ],
        ),
        migrations.AlterUniqueTogether(
            name='sharingsite',
            unique_together=set([('hostname', 'port')]),
        ),
    ]



from __future__ import absolute_import, unicode_literals

import django
import os


ALLOWED_HOSTS = ['*']

SECRET_KEY = 'not needed'

ROOT_URLCONF = 'wagtailsharing.tests.urls'

DATABASES = {
    'default': {
        'ENGINE': os.environ.get(
            'DATABASE_ENGINE',
            'django.db.backends.sqlite3'
        ),
        'NAME': os.environ.get('DATABASE_NAME', 'wagtailsharing.sqlite'),
        'USER': os.environ.get('DATABASE_USER', None),
        'PASSWORD': os.environ.get('DATABASE_PASS', None),
        'HOST': os.environ.get('DATABASE_HOST', None),

        'TEST': {
            'NAME': os.environ.get('DATABASE_NAME', None),
        },
    },
}

if django.VERSION >= (1, 10):
    MIDDLEWARE = (
        'django.middleware.common.CommonMiddleware',
        'django.contrib.sessions.middleware.SessionMiddleware',
        'django.middleware.csrf.CsrfViewMiddleware',
        'django.contrib.auth.middleware.AuthenticationMiddleware',
        'django.contrib.auth.middleware.SessionAuthenticationMiddleware',
        'django.contrib.messages.middleware.MessageMiddleware',
        'django.middleware.clickjacking.XFrameOptionsMiddleware',

        'wagtail.wagtailcore.middleware.SiteMiddleware',
    )
else:
    MIDDLEWARE_CLASSES = (
        'django.middleware.common.CommonMiddleware',
        'django.contrib.sessions.middleware.SessionMiddleware',
        'django.middleware.csrf.CsrfViewMiddleware',
        'django.contrib.auth.middleware.AuthenticationMiddleware',
        'django.contrib.auth.middleware.SessionAuthenticationMiddleware',
        'django.contrib.messages.middleware.MessageMiddleware',
        'django.middleware.clickjacking.XFrameOptionsMiddleware',

        'wagtail.wagtailcore.middleware.SiteMiddleware',
    )

INSTALLED_APPS = (
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.staticfiles',

    'taggit',

    'wagtail.contrib.modeladmin',
    'wagtail.contrib.settings',
    'wagtail.tests.testapp',
    'wagtail.wagtailadmin',
    'wagtail.wagtailcore',
    'wagtail.wagtaildocs',
    'wagtail.wagtailforms',
    'wagtail.wagtailimages',
    'wagtail.wagtailsites',
    'wagtail.wagtailusers',

    'wagtailsharing',
)

STATIC_URL = '/static/'

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
                'django.template.context_processors.request',
            ],
            'debug': True,
        },
    },
]

WAGTAIL_SITE_NAME = 'Test Site'

WAGTAILADMIN_RICH_TEXT_EDITORS = {
    'default': {
        'WIDGET': 'wagtail.wagtailadmin.rich_text.HalloRichTextArea'
    },
    'custom': {
        'WIDGET': 'wagtail.tests.testapp.rich_text.CustomRichTextArea'
    },
}



from __future__ import absolute_import, unicode_literals

from django.conf.urls import url
from wagtail.wagtailcore.urls import (
    serve_pattern, urlpatterns as wagtailcore_urlpatterns
)

from wagtailsharing.views import ServeView


urlpatterns = [
    url(serve_pattern, ServeView.as_view(), name='wagtail_serve')
    if urlpattern.name == 'wagtail_serve' else urlpattern
    for urlpattern in wagtailcore_urlpatterns
]



from django.db import IntegrityError
from django.test import RequestFactory, TestCase
from wagtail.wagtailcore.models import Site

from wagtailsharing.models import SharingSite


class TestSharingSite(TestCase):
    def setUp(self):
        self.default_site = Site.objects.get(is_default_site=True)
        self.factory = RequestFactory()

    def create(self, **kwargs):
        return SharingSite.objects.create(site=self.default_site, **kwargs)

    def test_can_create(self):
        self.create(hostname='hostname', port=8000)

    def test_site_related_name_default_empty(self):
        with self.assertRaises(SharingSite.DoesNotExist):
            self.default_site.sharing_site

    def test_site_related_name(self):
        self.create(hostname='hostname', port=8000)
        self.assertIsNotNone(self.default_site.sharing_site)

    def test_str(self):
        site = self.create(hostname='hostname', port=8000)
        self.assertEqual(str(site), 'hostname:8000')

    def test_str_no_port(self):
        site = self.create(hostname='hostname')
        self.assertEqual(str(site), 'hostname')

    def test_uniqueness(self):
        self.create(hostname='hostname', port=1234)
        with self.assertRaises(IntegrityError):
            self.create(hostname='hostname', port=1234)

    def test_multiple_sharing_sites_not_allowed(self):
        self.create(hostname='hostname', port=1234)
        with self.assertRaises(IntegrityError):
            self.create(hostname='otherhost', port=5678)

    def test_find_for_request(self):
        sharing_site = self.create(hostname='hostname', port=1234)
        request = self.factory.get('/', HTTP_HOST='hostname', SERVER_PORT=1234)
        self.assertEqual(SharingSite.find_for_request(request), sharing_site)

    def test_find_for_request_no_sites(self):
        request = self.factory.get('/', HTTP_HOST='hostname', SERVER_PORT=1234)
        with self.assertRaises(SharingSite.DoesNotExist):
            SharingSite.find_for_request(request)

    def test_find_for_request_wrong_port(self):
        self.create(hostname='hostname', port=1234)
        request = self.factory.get('/', HTTP_HOST='hostname', SERVER_PORT=5678)
        with self.assertRaises(SharingSite.DoesNotExist):
            SharingSite.find_for_request(request)

    def test_find_for_request_hostname_keyerror(self):
        request = self.factory.get('/', SERVER_PORT=5678)
        del request.META['SERVER_NAME']

        with self.assertRaises(SharingSite.DoesNotExist):
            SharingSite.find_for_request(request)

    def test_root_url_port_80_http_no_port(self):
        site = SharingSite(hostname='test.hostname', port=80)
        self.assertEqual(site.root_url, 'http://test.hostname')

    def test_root_url_port_443_https(self):
        site = SharingSite(hostname='test.hostname', port=443)
        self.assertEqual(site.root_url, 'https://test.hostname')

    def test_root_url_other_port_http(self):
        site = SharingSite(hostname='test.hostname', port=1234)
        self.assertEqual(site.root_url, 'http://test.hostname:1234')



from wagtail.wagtailcore.models import Site

from wagtailsharing.models import SharingSite


def get_sharing_url(page):
    """Get a sharing URL for the latest revision of a page, if available."""
    url_parts = page.get_url_parts()

    if url_parts is None:
        # Page is not routable.
        return None

    site_id, root_url, page_path = url_parts

    site = Site.objects.get(id=site_id)

    try:
        sharing_site = site.sharing_site
    except SharingSite.DoesNotExist:
        # Site is not shared.
        return None

    return sharing_site.root_url + page_path



from setuptools import find_packages, setup


install_requires = [
    'Django>=1.8,<1.11',
    'wagtail>=1.6,<1.9',
]


testing_extras = [
    'coverage>=3.7.0',
    'flake8>=2.2.0',
    'mock>=1.0.0',
]


short_description = 'Easier sharing of Wagtail drafts'


setup(
    name='wagtail-sharing',
    url='https://github.com/cfpb/wagtail-sharing',
    author='CFPB',
    author_email='tech@cfpb.gov',
    license='CCO',
    version='0.4',
    include_package_data=True,
    packages=find_packages(),
    install_requires=install_requires,
    extras_require={
        'testing': testing_extras,
    },
    description=short_description,
    long_description=open('README.rst').read(),
)



import wagtail.wagtailcore.urls

import wagtailsharing.urls

from django.conf.urls import url
from django.test import TestCase
from mock import patch


try:
    from importlib import reload
except ImportError:
    pass


class TestUrlPatterns(TestCase):
    def setUp(self):
        def test_view():
            pass  # pragma: no cover

        root_patterns = [
            url(r'^foo/$', url, name='foo'),
            url(r'^((?:[\w\-]+/)*)$', url, name='wagtail_serve'),
            url(r'^bar/$', url, name='bar'),
        ]

        self.patcher = patch.object(
            wagtail.wagtailcore.urls,
            'urlpatterns',
            root_patterns
        )
        self.patcher.start()
        self.addCleanup(self.patcher.stop)

        reload(wagtailsharing.urls)
        self.urlpatterns = wagtailsharing.urls.urlpatterns

    def test_leaves_previous_urls_alone(self):
        self.assertEqual(self.urlpatterns[0].name, 'foo')
        self.assertEqual(self.urlpatterns[0].regex.pattern, r'^foo/$')

    def test_replaces_wagtail_serve(self):
        self.assertEqual(self.urlpatterns[1].name, 'wagtail_serve')
        self.assertEqual(self.urlpatterns[1].callback.__name__, 'ServeView')

    def test_leaves_later_urls_alone(self):
        self.assertEqual(self.urlpatterns[2].name, 'bar')
        self.assertEqual(self.urlpatterns[2].regex.pattern, r'^bar/$')



from __future__ import absolute_import, unicode_literals

from django.utils.translation import ugettext_lazy as _
from wagtail.wagtailadmin import widgets as wagtailadmin_widgets
from wagtail.contrib.modeladmin.options import ModelAdmin, modeladmin_register
from wagtail.wagtailcore import hooks

from wagtailsharing.helpers import get_sharing_url
from wagtailsharing.models import SharingSite


class SharingSiteModelAdmin(ModelAdmin):
    model = SharingSite
    menu_icon = 'site'
    menu_order = 603
    add_to_settings_menu = True
    list_display = ('site', 'hostname', 'port')


modeladmin_register(SharingSiteModelAdmin)


@hooks.register('register_page_listing_more_buttons')
def add_sharing_link(page, page_perms, is_parent=False):
    sharing_url = get_sharing_url(page)

    if sharing_url:
        if hasattr(page, 'get_admin_display_title'):
            title = page.get_admin_display_title()
        else:
            title = page.title

        yield wagtailadmin_widgets.Button(
            'View sharing link',
            sharing_url,
            attrs={
                'title': _("View shared revision of '{}'").format(title),
            },
            priority=90
        )
